use std::path::{Path, PathBuf};

use anyhow::{Context, Result};
use sequoia_openpgp::{packet::Signature, serialize::SerializeInto, Cert};

pub(crate) struct Paths {
    pub(crate) spec: PathBuf,
    pub(crate) secret: PathBuf,
    pub(crate) public: PathBuf,
    pub(crate) rev: PathBuf,
}

impl Paths {
    pub(crate) fn new(base: PathBuf) -> Paths {
        let mut spec = base.clone();
        spec.push("spec.yml");
        let mut secret = base.clone();
        secret.push("secret.asc");
        let mut public = base.clone();
        public.push("public.asc");
        let mut rev = base;
        rev.push("rev.asc");
        Paths {
            spec,
            secret,
            public,
            rev,
        }
    }
    pub(crate) fn write(&self, cert: Cert, rev: Signature) -> Result<()> {
        self.write_part(Self::public, cert.armored())
            .context("Failed to write public certificate to file")?;
        self.write_part(Self::secret, cert.as_tsk().armored())
            .context("Failed to certificate including private keys to file")?;
        self.write_part(
            Self::rev,
            cert.insert_packets(rev)
                .context("Failed to write revocation signature packet into certificate")?
                .armored(),
        )
        .context("Failed to write revoked certificate to file")?;
        Ok(())
    }

    fn write_part<F, S>(&self, path_selector: F, data: S) -> Result<()>
    where
        F: Fn(&Self) -> &Path,
        S: SerializeInto,
    {
        std::fs::write(
            path_selector(self),
            SerializeInto::to_vec(&data)
                .context("Failed to serialize certificate into byte vec")?,
        )
        .context("Failed to write serialized certificate into file")?;
        Ok(())
    }

    fn public(&self) -> &Path {
        self.public.as_path()
    }

    fn secret(&self) -> &Path {
        self.secret.as_path()
    }

    fn rev(&self) -> &Path {
        self.rev.as_path()
    }
}
